#!/usr/bin/env python
import psycopg2
from datetime import datetime, timedelta
import sys
sys.path.append('/opt/logserver/db')
import maintain_database
from nose.tools import ok_

def create_connection():
    try:
        conn = psycopg2.connect("dbname=op5lsu user=op5lsu")
    except:
        print "Unable to connect to database"
    return conn

def clean_db(conn):
    cur = conn.cursor()
    cur.execute("select table_name from information_schema.tables \
                where table_catalog = 'op5lsu' \
                and table_name like '%messages_%';")
    for table in cur.fetchall():
        cur.execute("DROP TABLE %s;" % table[0])
    conn.commit()
    cur.close()

def test_get_messages_tables():
    expected_tables = []
    actual_tables = []

    conn = create_connection()
    clean_db(conn)
    maintain_database.create_tables(conn, '2014-12-31 23:00:00')

    cur = conn.cursor()
    cur.execute("select table_name from information_schema.tables \
                where table_catalog = 'op5lsu' \
                and table_name like '%messages_%';")

    for table in cur.fetchall():
        expected_tables.append(table[0])

    for table in maintain_database.get_messages_tables(conn):
        actual_tables.append(table[0])

    actual_tables.sort()
    expected_tables.sort()


    ok_(actual_tables == expected_tables, 'Table mismatch: %s != %s' % (actual_tables, expected_tables))


def test_create_tables_from_now():
    expected_tables = []
    actual_tables = []

    conn = create_connection()
    clean_db(conn)

    for i in range(0, 3):
        tmp_date = datetime.now() + timedelta(hours=i)
        expected_tables.append(tmp_date.strftime('messages_%Y_%m_%d_%H'))

    maintain_database.create_tables(conn)

    for table in maintain_database.get_messages_tables(conn):
        actual_tables.append(table[0])

    actual_tables.sort()
    expected_tables.sort()


    ok_(actual_tables == expected_tables, 'Table mismatch: %s != %s' % (actual_tables, expected_tables))

def test_create_tables_from_time_around_day_and_year():
    expected_tables = ['messages_2014_12_31_23', 'messages_2015_01_01_00', 'messages_2015_01_01_01']
    actual_tables = []

    conn = create_connection()
    clean_db(conn)
    maintain_database.create_tables(conn, '2014-12-31 23:00:00')

    for table in maintain_database.get_messages_tables(conn):
        actual_tables.append(table[0])

    actual_tables.sort()
    expected_tables.sort()


    ok_(actual_tables == expected_tables, 'Table mismatch: %s != %s' % (actual_tables, expected_tables))

def test_get_keep_in_db_setting():
    conn = create_connection()

    cur = conn.cursor()
    cur.execute('UPDATE settings SET keep_in_db = 5')
    conn.commit()

    keep_in_db = maintain_database.get_keep_in_db_setting(conn)

    ok_(keep_in_db == 5, 'Setting mismatch: %s != 5' % keep_in_db)

def test_rotate_db_5h_value():
    expected_tables = []
    actual_tables = []

    conn = create_connection()
    clean_db(conn)

    cur = conn.cursor()
    cur.execute('UPDATE settings SET keep_in_db = 5')
    conn.commit()

    for i in range(-5, 3):
        tmp_date = datetime.now() + timedelta(hours=i)
        expected_tables.append(tmp_date.strftime('messages_%Y_%m_%d_%H'))

    maintain_database.create_tables(conn)

    #Also create 8 hours earlier tables
    for i in range(0, 9):
        tmp_date = datetime.now() - timedelta(hours=i)
        maintain_database.create_tables(conn, tmp_date.strftime('%Y-%m-%d %H:00:00'))

    maintain_database.rotate_db(conn)

    for table in maintain_database.get_messages_tables(conn):
        actual_tables.append(table[0])

    actual_tables.sort()
    expected_tables.sort()


    ok_(actual_tables == expected_tables, 'Table mismatch: %s != %s' % (actual_tables, expected_tables))

def test_rotate_db_1h_value():
    expected_tables = []
    actual_tables = []

    conn = create_connection()
    clean_db(conn)

    cur = conn.cursor()
    cur.execute('UPDATE settings SET keep_in_db = 1')
    conn.commit()

    for i in range(-1, 3):
        tmp_date = datetime.now() + timedelta(hours=i)
        expected_tables.append(tmp_date.strftime('messages_%Y_%m_%d_%H'))

    maintain_database.create_tables(conn)

    #Also create 8 hours earlier tables
    for i in range(0, 9):
        tmp_date = datetime.now() - timedelta(hours=i)
        maintain_database.create_tables(conn, tmp_date.strftime('%Y-%m-%d %H:00:00'))

    maintain_database.rotate_db(conn)

    for table in maintain_database.get_messages_tables(conn):
        actual_tables.append(table[0])

    actual_tables.sort()
    expected_tables.sort()

    ok_(actual_tables == expected_tables, 'Table mismatch: %s != %s' % (actual_tables, expected_tables))

